if select(2, UnitClass("player")) ~= "DEATHKNIGHT" then
   return
end

------------------------------------------------------------------------
local BG_Tex = "Interface\\AddOns\\ThreeRuneBars\\diseases.tga";
local FF_Icon = "Interface\\Icons\\Spell_DeathKnight_FrostFever";
local BP_Icon = "Interface\\Icons\\Spell_DeathKnight_BloodPlague";


TRB_Diseases = TRB_Module:create("Diseases");

function TRB_Diseases:OnDisable()
	self.frame:SetScript("OnEvent", nil);
	self.frame:SetScript("OnUpdate", nil);
end

function TRB_Diseases:getDefault(val)
	if( val == "Position" ) then
		return { "CENTER", nil, "CENTER", 160, -25 };
	end
	return nil;
end

function TRB_Diseases:OnEnable()
	if( not self.frame ) then
		local f = CreateFrame("frame", nil, UIParent);
	
		-- Set Position and size
		f:ClearAllPoints();
		f:SetWidth(122);
		f:SetHeight(40);
		f:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
		f:SetFrameStrata("HIGH"); -- Set to HIGH framestrata to be visible ontop of Blizzards "Power Aura" thing
		f.owner = self;
		
		-- Background
		local bg = f:CreateTexture(nil, "BACKGROUND");
		bg:SetWidth(128);
		bg:SetHeight(128);
		bg:SetPoint("CENTER", f, "CENTER", 0, 0);
		bg:SetTexture(BG_Tex);
		f.bg = bg;
		
		self.frame = f;
		
		self:CreateMoveFrame();
	end

	if( not self.Bars ) then

		local ff = self:CreateBar("Frost_Fever", self.frame);
		local bp = self:CreateBar("Blood_Plague", self.frame);
		
		ff:SetValue(0);
		bp:SetValue(0);
		
		ff:SetWidth(98);
		bp:SetWidth(98);
		ff:SetStatusBarColor( unpack(TRB_Config[self.name].Colors["ff"]) );
		bp:SetStatusBarColor( unpack(TRB_Config[self.name].Colors["bp"]) );
		ff:Show();
		bp:Show();
		
		ff.id = "Frost Fever";
		bp.id = "Blood Plague";
		
		ff:SetPoint("TOPLEFT", self.frame, "TOPLEFT", 22, -6);
		bp:SetPoint("TOPLEFT", self.frame, "TOPLEFT", 22, -26);
		
		-- Text
		local text = ff:CreateFontString(nil, "ARTWORK", "GameFontHighlightExtraSmall");
		text:SetPoint("CENTER", ff, "CENTER", 0, 0);
		text:SetText();
		ff.text = text;
		text = bp:CreateFontString(nil, "ARTWORK", "GameFontHighlightExtraSmall");
		text:SetPoint("CENTER", bp, "CENTER", 0, 0);
		text:SetText();
		bp.text = text;
		
		-- Create Icons
		ff.icon = self.frame:CreateTexture(nil, "OVERLAY"); ff.icon:ClearAllPoints(); ff.icon:SetTexture(FF_Icon);
		ff.icon:SetPoint("TOPLEFT", self.frame, "TOPLEFT", 1, -1);
		ff.icon:SetWidth(18);
		ff.icon:SetHeight(18);
		
		bp.icon = self.frame:CreateTexture(nil, "OVERLAY"); bp.icon:ClearAllPoints(); bp.icon:SetTexture(BP_Icon);
		bp.icon:SetPoint("TOPLEFT", self.frame, "TOPLEFT", 1, -21);
		bp.icon:SetWidth(18);
		bp.icon:SetHeight(18);
		
		-- Bars
		self.Bars = {};
		self.Bars["ff"] = ff;
		self.Bars["bp"] = bp;
	end

	self.frame:RegisterEvent("UNIT_AURA");
	self.frame:RegisterEvent("PLAYER_TARGET_CHANGED");

	self.last = 0;
	self.frame:SetScript("OnEvent", function(frame, event, ...) frame.owner[event](frame.owner, ...); end );
	self.frame:SetScript("OnUpdate", function(frame, elapsed) frame.owner:OnUpdate(elapsed); end );
	
	self.frame:Hide();
end

function TRB_Diseases:UpdateDiseaseBar(disease)
		local bar = self.Bars[disease];

		local name, _, icon, _,_,  duration, expirationTime, _ = UnitAura("target", bar.id, nil, "PLAYER|HARMFUL");
		
		if name then
			local val = expirationTime - GetTime();
			local valT = format("%.0f", val);
			if( val < 5 ) then valT = format("%.1f", val); end
			if( val < 0 ) then valT = ""; end;
			local val = (val / duration) * 100;
			bar:SetValue(val);
			
			bar.text:SetText( valT );
--			bar:Show();
		else
--			bar:Hide();
			bar:SetValue(0);
			bar.text:SetText();
		end
end

function TRB_Diseases:showBars(v)
	if(v) then
		self.frame:Show();
		self.needUpdate = true;
	else
		self.frame:Hide();
		--self:SetFrameStrata("MEDIUM");
		self.needUpdate = false;
		self.Bars["ff"]:SetValue(0);
		self.Bars["bp"]:SetValue(0);
	end
end

function TRB_Diseases:UNIT_AURA(unit)
end

function TRB_Diseases:PLAYER_TARGET_CHANGED()
	if( UnitExists("target")) then
		self:showBars(true);
	else
		self:showBars(false);
	end
end

function TRB_Diseases:OnUpdate(elapsed)
	self.last = self.last + elapsed;
	
	if( self.last > 0.01 ) then
		if( self.needUpdate) then
			self:UpdateDiseaseBar("ff");
			self:UpdateDiseaseBar("bp");
		end
		
		self.last = 0;
	end
end


-- Register Disease module
ThreeRuneBars:RegisterModule(TRB_Diseases.name, TRB_Diseases);

